#ifndef CREATURE_H
#define CREATURE_H

#include "RedBlack.h"
#include "IdSet.h"
#include "LnList.h"

#include "misc.h"
#include "Graphics.h"

class Dungeon;
class Object;

#define CREATURE_NAME_LEN 64 // belongs in CreatureAddon.h

class Skill
{
	public:
		Skill( const char *, int );
		
		bool Test();
		bool operator == ( const char * );
		
		status_t Save( BFile * );
		status_t Load( BFile * );

	protected:
		friend class InformationView;
		char name[CREATURE_NAME_LEN];
		int skill;
};

class SkillSet
{
	public:
		SkillSet();
		~SkillSet();
		
		void		Add( Skill * );
		Skill *		Find( const char * );
		Skill *		SkillAt( int );
		
		status_t Save( BFile * );
		status_t Load( BFile * );
		
	protected:
		List<Skill> skills;
};

class Creature;

class damage
{
	public:
		damage();
		
		int amount;
		int type;
		Creature * source;
		
		enum {
			SLASH,
			PIERCE,
			CRUSH,
			HEAT,
			COLD,
			ELECTRICITY
		};
};

class Creature
{ // base class for creatures (and player)
	public:
		enum {
			BACK		= 0x00000001,
			HEAD		= 0x00000002,
			LEFT_ARM	= 0x00000004,
			RIGHT_ARM	= 0x00000008,
			TORSO		= 0x00000010,
			LEFT_HAND	= 0x00000020,
			RIGHT_HAND	= 0x00000040,
			LEFT_LEG	= 0x00000080,
			RIGHT_LEG	= 0x00000100,
			LEFT_FOOT	= 0x00000200,
			RIGHT_FOOT	= 0x00000400
		};
		class Stat
		{
			public:
				int base;
				int curr;
		};
		
		Creature( const char *, const char * _internal = NULL );
		virtual ~Creature();

		virtual void AI(); // Let some AI have the critter do something
		virtual void Damage( damage );
		virtual void Death();
		virtual void Attack( Creature * );
		
		bool TestSkill( const char * );
		void DropObject( uint32 );
		bool PickUpObject( uint32 );
		
		void TurnToFace( Pos );
		
		Object * FindEquiped( uint32 ); // equip-mask

		typedef struct {
			uint32		id;
			char		bitmap[64];
			Pos			pos;
			Stat		INT, STR, HP;
			direction	facing;
		} Info;
		
		IdSet		inventory; // all things carried
		IdSet		equipment; // currently equiped
		SkillSet	skills;
		
		char		name[CREATURE_NAME_LEN];
		const char	* internal_name;
		
		Info		info;
		
		virtual status_t	Load( BFile * );
		virtual status_t	Save( BFile * );

	protected:
		void MoveTo( Pos );
		
		bool FindPath( Pos );
		void FollowPath();
		bool HasPath();
		
	private:
		List<Pos>	* path;
};

class critter_info
{
	public:
		critter_info();
		
		const critter_info & operator = ( const critter_info & );
		bool operator != ( const critter_info & );
		bool operator < ( const critter_info & );
		
		Creature * creature;
};

class CreatureInfoSet; // for load

class CreatureSet
{
	protected:
		uint32 next_id;
		
		List<Creature> list;
		
	public:
		CreatureSet();
		~CreatureSet();
		
		void		Add( Creature *, bool set_id = true );
		void		Remove( Creature * );
		Creature *	Find( uint32 );
		void		DoAI( int level );
		
		status_t	Load( BFile *, CreatureInfoSet * );
		status_t	Save( BFile * );
		void		Clear();
};

#endif
