#ifndef DUNGEON_H
#define DUNGEON_H

#include <OS.h>
#include <Locker.h>
#include <View.h>

#include "CreatureAddon.h"
#include "ObjectAddon.h"
#include "Graphics.h"
#include "LnList.h"
#include "LnFifo.h"

#include "Object.h"
#include "IdSet.h"
#include "Creature.h"
#include "misc.h"

class Dungeon;

class Tile
{
	public:
		Tile();
		
		typedef struct {
			char		bitmap[24];
			uint32		flags;
			uint32		creature;
		} TileInfo;
		
		IdSet		objects;
		
		TileInfo	info;
		
//		Dungeon 	* dungeon;
		
		void Draw( BView *, BPoint );
		
		status_t Save( BFile * );
		status_t Load( BFile * );
};

class Room
{
	public:
		int ent_x, ent_y;
		direction dir;
};

class Dungeon
{
	public:
		Dungeon();
		~Dungeon();
		
		void 		Lock();
		void 		Unlock();

		void		Setup( Graphics*, CreatureInfoSet*, ObjectInfoSet*,
						CreatureSet *, ObjectSet *
					);

		void 		Generate(Creature *, int level, int xs, int ys); // size; x,y

		// Load and save data specific to this dungeon at current position in a file
		status_t	Load( BFile * );
		status_t 	Save( BFile * );
		
		Tile *		TileAt( Pos );

		bool		MoveCreature( uint32, Pos, Pos ); // id; from; to
		Creature *	FindCreature( uint32 );

		Object *	FindObject( uint32 );
		void		AddObject( Object *, Pos ); // thing, x,y
		void		AddObject( Object * ); // thing, add only to list not map
		Object *	CreateObject( const char * ); // create and add to list
		
		void		DoLOS(Pos, float, uint32, direction = ALL); // x,y, range, flags
		List<Pos> *	FindPath( Pos, Pos, int max_steps=50 ); // from, to
		
		void		ClearTileFlags(uint32);

		void		DoAI(); // call Creature::AI() for all creatures in dungeon
		
		bool		CastRay(Pos, Pos, uint32);// from, to, flag
//		bool		CastRay(FloatPos, FloatPos, uint32);// from, to, flag
		
		int			Width();
		int			Height();
		int			Level();
		
	protected:
		class DungeonInfo
		{
			public:
				Version		version;
				int			mapsize_x, mapsize_y;
				int			level;
				
				DungeonInfo( Version );
		};
		
		friend		class Tile;
		friend		class Creature;
		friend		class PlayerView;
		friend		class IconView;
		friend		class GroundView;
				
		void			CheckLock(); // check if locked, if not exit app with warning
		BLocker			lock;

		DungeonInfo		info;
		Tile			* map;

		Graphics		* graphics;
		CreatureInfoSet	* creature_info;
		ObjectInfoSet	* object_info;
		
		CreatureSet		* creatures;
		ObjectSet		* objects;
		
		bool		CreateRoom( Room, FIFO<Room> &);
};

extern Dungeon * current_dungeon;

#endif
