#ifndef PLAYER_VIEW_H
#define PLAYER_VIEW_H

#include <View.h>
#include <StringView.h>

#include "Dungeon.h"

class object_pack
{
	public:
		object_pack();
		
		BBitmap		* bitmap;
		uint32		id; // object id
};

enum {
	DRAG_OBJECT_MSG = 'doMG',
	UPDATE			= 'upDt',
	REDRAW			= 'reDr'
};

class IconView : public BView
{
	public:
		enum {
			EQUIPMENT, INVENTORY, GROUND
		}; // type
		
		IconView( BPoint, int32 _type, uint32 _mask = 0xffffffff);
		virtual ~IconView();
		
		virtual void Draw( BRect );
		virtual void MessageReceived( BMessage * );
		virtual void MouseDown( BPoint );
				
		void SetObject( object_pack );

	protected:
		object_pack	object;
		Dungeon		* dungeon;
		int32		type;
		uint32		mask;
};

class StatView : public BView
{
	public:
		StatView( BRect, const char * );
		
		void SetStat( Creature::Stat );
		
		virtual void Draw( BRect );
	
	protected:
		Creature::Stat stat;
		rgb_color bg_col, tx_col;
		char text[64];
		const char * name;
};

class InventoryView : public BView
{
	public:
		InventoryView( BRect );
		virtual ~InventoryView();

		virtual void AttachedToWindow();

	protected:
		BBitmap		* m_background;
		
		IconView	* head;
		IconView	* torso;
		IconView	* left_hand, * right_hand;
		IconView	* left_foot, * right_foot;

		IconView	* inventory[9];
};

class InformationView : public BView
{
	public:
		InformationView( BRect );
		virtual ~InformationView();
		
		virtual void AttachedToWindow();
		virtual void Draw( BRect );
	
	protected:
		BBitmap		* m_background;
};

class PlayerView : public BView
{
	public:
		PlayerView(BRect );
		virtual ~PlayerView();
		
		virtual void AttachedToWindow();
		virtual void MessageReceived( BMessage * );
		virtual void Draw( BRect );
		virtual void MouseDown( BPoint );
		
		void Update();
		void Redraw();

	protected:
		BView			* m_curr_view;
		InformationView * m_info;
		InventoryView	* m_inventory;
		
		BBitmap		* m_background;
		
		StatView	* stat_hp;

		StatView	* stat_str;
		StatView	* stat_int;
		
		IconView	* ground[9];
};

#endif

