#ifndef MISC_H
#define MISC_H

#include <Messenger.h>
#include <Font.h>

extern BMessenger game_messenger; // in MainWindow.cpp
enum {
	ROGUE_MSG = 'rgMS',
	SAVE_REQUEST = 'rgSV',
	LOAD_REQUEST = 'rgLO',
	ANIMATION = 'rgAN'
};

enum  direction {
	UP = 0, UP_RIGHT, RIGHT, DOWN_RIGHT, DOWN, DOWN_LEFT, LEFT, UP_LEFT,
	ALL // full circle, used for LOS
};

enum {
	black_bitmap = 1,
	grey_bitmap,
	wall_front_bitmap,
	wall_compact_bitmap,
	floor_bitmap,
	stairs_up_bitmap,
	stairs_down_bitmap,
	door_open_bitmap,
	door_closed_bitmap
};

enum {
	WALL		= 0x00000001,
	FLOOR		= 0x00000002,
	CREATURE	= 0x00000004,
	OBJECT		= 0x00000008,
	DOOR		= 0x00000010,
	EXPLORED	= 0x00000020, // play has visited this tile, display as 'shaded'
	VISIBLE		= 0x00000040, // player can see this tile
	STAIRS_UP	= 0x00000080,
	STAIRS_DOWN	= 0x00000100,
	DOOR_OPEN	= 0x00000200,
		
	INVALID_ID = 0, // this means "if (tile.creature) { display_creature(..) };"
	
	TILE_WIDTH	= 32,	// for graphics
	TILE_HEIGHT = 32
};

enum { // commong error codes
	LN_INVALID_VERSION	= -4001,
	LN_IO_ERROR			= -4002
};

class Pos
{
	public:
		Pos();
		Pos(int,int,int);
		Pos(const Pos &);
		
		bool operator == (const Pos &);
		bool operator != (const Pos &);
		
		Pos operator + ( direction );
		
		int x,y, level;
};

int _round( float );

class FloatPos
{
	public:
		FloatPos();
		FloatPos(float,float, int);
		FloatPos(const Pos &); // add 0.49 to each coordinate
		FloatPos(const FloatPos &);
		
		operator Pos ();
		
		bool operator == (const FloatPos &);
		
		const FloatPos & operator += ( const FloatPos & );
		
		float x,y;
		int level;
};

float _dist( Pos, Pos );

float _fabs( float );

extern rgb_color std_txt;

class TextMode
{
	public:
		enum mode {
			NORMAL,		// * 1.0
			BOLD		// * 1.5
		};
};

void game_message( Pos, const char *, TextMode::mode = TextMode::NORMAL, rgb_color col = std_txt );

void animation( Pos, const char * ); // map pos, bitmap

typedef struct {
	const char	* file;
	uint32		id;
} bitmap_info;


class Version 
{
	public:
		Version( int, int );
		Version( const Version & );
		
		bool operator <  ( const Version & );
		bool operator != ( const Version & );
		
		status_t	Save( BFile * );
		status_t	Load( BFile * );
		
	protected:
		int major, minor;
};

class Graphics;
extern Graphics * g_graphics; // defined in SetupWindow.cpp

#endif
